import config

"""
...
Usage
---------
Sql = Sql('prefix name e.g. myprojectname')
Sql.write_to_*()

"""

class Sql:
    """
    A class used to represent the Sql container

    ...

    Attributes
    ----------
    prefix : str
        The prefix applied for all container names
    container_name : str
        the name to give the containr inside the dockerfile
    dockerfile_name : str
        a formatted string that defines the name of the docker file for sql

    Methods
    -------
    get_dockerfile_content()
        Returns the necessary content to the dockerfile for sql

    get_docker_compose_content()
        Returns to the docker compose file with the necessary sql content

    create_dump_file(root_path: str)
        Creates an empty dump file

    create_env_file(root_path: str)
        Creates an environmental file and adds the default credentials
    """

    def __init__(self, prefix_for_containers: str):
        """
        Parameters
        ----------
        prefix : str
            The prefix applied to all container names
        container_name : str
            the name to give the containr inside the dockerfile
        dockerfile_name : str
            a formatted string that defines the name of the docker file for sql
        """
        self.prefix: str = prefix_for_containers
        self.container_name: str = prefix_for_containers + '_sql'
        self.dockerfile_name: str = 'sql.dockerfile'

    def get_dockerfile_content(self):
        """
        Returns the neccessary content to the dockerfile for sql

        """

        dockerfile_content: List[str] = [
            'FROM mysql:5.6',
            '# Update and install required packages',
            'RUN apt-get update -y',
            '',
            'COPY    ./.docker/data/sql-data-dump.sql /docker-entrypoint-initdb.d/'
        ]
        return dockerfile_content

    def create_dump_file(self, root_path: str):
        """
       Creates an empty dump file

        """
        open('{}/.docker/data/sql-data-dump.sql'.format(root_path), 'x').close()

    def create_env_file(self, root_path: str):
        """
        Creates an environmental file with default credentials

        """
        content = [
            'MYSQL_ROOT_PASSWORD=rootpassword',
            'MYSQL_DATABASE=db',
            'MYSQL_USER=user',
            'MYSQL_PASSWORD=password'
        ]
        file = open('{}/.docker/env/sql.env'.format(root_path), 'x')
        for text in content:
            file.write(text + '\n')

    def get_docker_compose_content(self):
        """
        Returns the necessary content to the docker-compose.yml file for sql

        """

        docker_compose_content: List[str] = [
            "  sql:",
            "    container_name: {}".format(self.container_name),
            "    build:",
            "      context: .",
            "      dockerfile: .docker/{}".format(self.dockerfile_name),
            "    ports:",
            '      - "3007:3006"',
            "    env_file:",
            "      - ./.docker/env/sql.env",
            "    networks:",
            "      - {}-network".format(self.prefix)
        ]
        return docker_compose_content


import config

"""
...
Usage
---------
deno = Deno('prefix name e.g. myprojectname')
deno.write_to_*()

"""

class Deno:
    """
    A class used to represent the Deno container

    ...

    Attributes
    ----------
    prefix : str
        The prefix applied for all container names
    container_name : str
        the name to give the container inside the dockerfile
    dockerfile_name : str
        a formatted string that defines the name of the docker file for node

    Methods
    -------
    get_dockerfile_content()
        Gets the necessary content for the dockerfile for deno

    get_docker_compose_content()
        Gets to the docker compose content for deno
    """

    def __init__(self, prefix_for_containers: str):
        """
        Parameters
        ----------
        prefix : str
            The prefix applied to all container names
        container_name : str
            the name to give the container inside the dockerfile
        dockerfile_name : str
            a formatted string that defines the name of the docker file for deno
        """
        self.prefix: str = prefix_for_containers
        self.container_name: str = prefix_for_containers + '_deno'
        self.dockerfile_name: str = 'deno.dockerfile'
        self.port = config.ports['deno']

    def get_dockerfile_content(self):
        """
        Writes the necessary content to the dockerfile for deno

        """

        dockerfile_content: List[str] = [
            'FROM debian:stable-slim',
            '',
            'RUN apt update -y',
            'RUN apt install bash curl unzip -y',
            '',
            'RUN curl -fsSL https://deno.land/x/install/install.sh | DENO_INSTALL=/usr/local sh -s v0.39.0',
            'RUN export DENO_INSTALL="/root/.local"',
            'RUN export PATH="$DENO_INSTALL/bin:$PATH"'
        ]
        return dockerfile_content

    def get_docker_compose_content(self):
        """
        Returns the necessary content for the docker-compose.yml file for node

        """

        docker_compose_content: List[str] = [
            "  drash:",
            "    container_name: {}".format(self.container_name),
            "    build:",
            "      context: .",
            "      dockerfile: .docker/{}".format(self.dockerfile_name),
            "    volumes:",
            "      - ./src:/var/www/src",
            "    working_dir: /var/www/src",
            "    ports:",
            '      - "{}:{}"'.format(self.port, self.port),
            '    command: bash -c "deno --allow-net app.ts"',
            "    networks:",
            "      - {}-network".format(self.prefix)
        ]
        return docker_compose_content


import unittest
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO
import sys
from contextlib import contextmanager
from unittest.mock import patch
from unittest import mock
from unittest import TestCase

@contextmanager
def captured_output():
    new_out, new_err = StringIO(), StringIO()
    old_out, old_err = sys.stdout, sys.stderr
    try:
        sys.stdout, sys.stderr = new_out, new_err
        yield sys.stdout, sys.stderr
    finally:
        sys.stdout, sys.stderr = old_out, old_err

from validate import Validate

class ValidateTest(unittest.TestCase):

    # Returns True or False.
    def test_dir_exists(self):
        # when dir exists
        try:
            Validate.dir_exists(".")
            self.assertTrue(False)
        except:
            self.assertTrue(True)

        # when dir doesn't exist
        exists = Validate.dir_exists("~/IdontExist")
        self.assertFalse(exists)

    def test_is_valid_dir_name(self):
        with captured_output() as (out, err):
            Validate.is_valid_dir_name('Test-Dir')
            output = out.getvalue().strip()
            self.assertEqual(output, '')

if __name__ == '__main__':
    unittest.main()


import config

"""
...
Usage
---------
redis = redis('prefix name e.g. myprojectname')
redis.write_to_*()

"""

class Redis:
    """
    A class used to represent the redis container

    ...

    Attributes
    ----------
    prefix : str
        The prefix applied for all container names
    container_name : str
        the name to give the containr inside the dockerfile
    port : int
        port for redis to run on
    dockerfile_name : str
        a formatted string that defines the name of the docker file for redis

    Methods
    -------
    get_dockerfile_content()
        Returns the necessary content to the dockerfile for redis

    get_docker_compose_content()
        Returns to the docker compose file with the neccessary redis content

    """

    def __init__(self, prefix_for_containers: str):
        """
        Parameters
        ----------
        prefix : str
            The prefix applied to all container names
        container_name : str
            the name to give the containr inside the dockerfile
        port : int
            port for redis to run on
        dockerfile_name : str
            a formatted string that defines the name of the docker file for redis
        """
        self.prefix: str = prefix_for_containers
        self.container_name: str = prefix_for_containers + '_redis'
        self.port: int = config.ports['redis']
        self.dockerfile_name: str = 'redis.dockerfile'

    def get_dockerfile_content(self):
        """
        Returns the necessary content to the dockerfile for redis

        """

        dockerfile_content: List[str] = [
            'FROM redis:5',
            'RUN apt update'
        ]
        return dockerfile_content

    def get_docker_compose_content(self):
        """
        Returns the necessary content to the docker-compose.yml file for redis

        """

        docker_compose_content: List[str] = [
            "  redis:",
            "    container_name: {}".format(self.container_name),
            "    build:",
            "      context: .",
            "      dockerfile: .docker/{}".format(self.dockerfile_name),
            "    ports:",
            "      - '{}:{}'".format(self.port, self.port),
            "    networks:",
            "      - {}-network".format(self.prefix)
        ]
        return docker_compose_content


import config

"""
...
Usage
---------
Nginx = Nginx('prefix name e.g. myprojectname')
Nginx.write_to_*()

"""

class Nginx:
    """
    A class used to represent the Nginx container

    ...

    Attributes
    ----------
    prefix : str
        The prefix applied for all container names
    container_name : str
        the name to give the containr inside the dockerfile
    port : int
        port for nginx to run on
    dockerfile_name : str
        a formatted string that defines the name of the docker file for nginx

    Methods
    -------
    get_dockerfile_content()
        Returns the neccessary content for the dockerfile for nginx

    get_docker_compose_content()
        Returns the docker compose content neccessary necessary for nginx

    write_to_config_file()
        returns the neccessary content for the config file with php-fpm support
    """

    def __init__(self, prefix_for_containers: str):
        """
        Parameters
        ----------
        prefix : str
            The prefix applied to all container names
        container_name : str
            the name to give the containr inside the dockerfile
        port : int
            port for nginx to run on
        dockerfile_name : str
            a formatted string that defines the name of the docker file for nginx
        """
        self.prefix: str = prefix_for_containers
        self.container_name: str = prefix_for_containers + '_nginx'
        self.port: int = config.ports['nginx']
        self.dockerfile_name: str = 'nginx.dockerfile'

    def get_dockerfile_content(self):
        """
        Holds the neccessary content for the dockerfile for nginx

        """

        dockerfile_content: List[str] = [
            'FROM nginx:latest',
            '# Update and install required packages',
            'RUN apt-get update',
            'RUN apt-get install vim -y',
            '',
            'COPY ./.docker/config/nginx.conf /etc/nginx/conf.d/nginx.conf',
            '',
            'ENTRYPOINT ["nginx"]',
            'CMD ["-g","daemon off;"]'
        ]
        return dockerfile_content

    def get_docker_compose_content(self):
        """
        Holds the neccessary content for the docker-compose.yml file for nginx

        """

        docker_compose_content: List[str] = [
            "  nginx:",
            "    container_name: {}".format(self.container_name),
            "    build:",
            "      context: .",
            "      dockerfile: .docker/{}".format(self.dockerfile_name),
            "    volumes:",
            "      - ./src:/var/www/src",
            "    working_dir: /var/www/src",
            "    ports:",
            "      - '{}:{}'".format(self.port, self.port),
            "    networks:",
            "      - {}-network".format(self.prefix)
        ]
        if self.depends_on_string != '':
            docker_compose_content.insert(2, self.depends_on_string)
        return docker_compose_content

    def get_config_file_content(self):
        """
        Holds the neccessary content for the config file for nginx with php-fpm support commented out
    
        """

        config_content: List[str] = [
            'server {',
	        '  listen {};'.format(self.port),
            '',
            '  ##',
            '  # PHP-FPM',
            '  ##',
            '  #location ~ \.php$ {',
  	        '    #include /etc/nginx/fastcgi_params;',
		    '    #root /var/www/src;',
            '    #fastcgi_split_path_info ^(.+?\.php)(/.*)$;',
            '    #fastcgi_pass	phpfpm:3002;',
		    '    #fastcgi_param SCRIPT_FILENAME $document_root/$fastcgi_script_name;',
            '  #}',
            '',
            '  location / {',
		    '    root /var/www/src;',
            '    index index.html;'
		    '    #index index.php;',
		    '    #rewrite ^ /index.php?$args last; break;',
	        '  }',
            '}'
        ]
        return config_content

    def update_services_to_depend_on(self, depends_on_string: str):
        self.depends_on_string = depends_on_string


import config
import os

"""
...
Usage
---------
mongoseeder = mongoseeder('prefix name e.g. myprojectname')
mongoseeder.write_to_*()

"""

class MongoSeeder:
    """
    A class used to represent the mongoseeder container

    ...

    Attributes
    ----------
    prefix : str
        The prefix applied for all container names
    container_name : str
        the name to give the containr inside the dockerfile
    dockerfile_name : str
        a formatted string that defines the name of the docker file for mongoseeder

    Methods
    -------
    get_dockerfile_content()
        Returns the necessary content to the dockerfile for mongoseeder

    get_docker_docker_compose_content()
        Returns to the docker compose file with the necessary mongoseeder content

    create_dump_dir()
        Creates a basic dump directory to seed from
    """

    def __init__(self, prefix_for_containers: str):
        """
        Parameters
        ----------
        prefix : str
            The prefix applied to all container names
        container_name : str
            the name to give the containr inside the dockerfile
        dockerfile_name : str
            a formatted string that defines the name of the docker file for mongoseeder
        """
        self.prefix: str = prefix_for_containers
        self.container_name: str = prefix_for_containers + '_mongoseeder'
        self.dockerfile_name: str = 'mongoseeder.dockerfile'

    def create_dump_dir(self, root_path: str):
        """
        Create the empty data dump directory
        """

        os.mkdir('{}/.docker/data/mongo-data-dump'.format(root_path))

    def get_dockerfile_content(self):
        """
        Returns the neccessary content to the dockerfile for mongoseeder

        """

        dockerfile_content: List[str] = [
            'FROM mongo',
            'COPY ./.docker/data/mongo-data-dump /mongo_data',
            'CMD mongorestore --host mongodb --db db /mongo_data'
        ]
        return dockerfile_content

    def get_docker_compose_content(self):
        """
        Returns the neccessary content to the docker-compose.yml file for mongoseeder

        """

        docker_compose_content: List[str] = [
            "  mongoseeder:",
            "    container_name: {}".format(self.container_name),
            "    build:",
            "      context: .",
            "      dockerfile: .docker/{}".format(self.dockerfile_name),
            "    depends_on:",
            "      - mongodb",
            "    networks:",
            "      - {}-network".format(self.prefix)
        ]
        return docker_compose_content


from response import Response
import unittest
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO
import sys
from contextlib import contextmanager
from unittest.mock import patch
from unittest import mock
from unittest import TestCase

@contextmanager
def captured_output():
    new_out, new_err = StringIO(), StringIO()
    old_out, old_err = sys.stdout, sys.stderr
    try:
        sys.stdout, sys.stderr = new_out, new_err
        yield sys.stdout, sys.stderr
    finally:
        sys.stdout, sys.stderr = old_out, old_err

class ResponseTest(unittest.TestCase):

    # Returns True or False.
    @patch('builtins.input', lambda *args: 'Hello')
    def test_ask_for_input(self):
        #mocked_input.side_effect = ['Albert Einstein', '42.81', 'done']
        result = Response.ask_for_input('hi')
        self.assertEqual(result, 'Hello')

    def test_show_info(self):
        with captured_output() as (out, err):
            Response.show_info('Test Info')
            output = out.getvalue().strip()
            self.assertEqual(output, '\x1b[92mTest Info\x1b[0m')

    def test_show_log(self):
        with captured_output() as (out, err):
            Response.show_log('Test Info')
            output = out.getvalue().strip()
            self.assertEqual(output, 'Test Info')

if __name__ == '__main__':
    unittest.main()


import config

"""
...
Usage
---------
Node = Node('prefix name e.g. myprojectname')
Node.write_to_*()

"""

class Node:
    """
    A class used to represent the Node container

    ...

    Attributes
    ----------
    prefix : str
        The prefix applied for all container names
    container_name : str
        the name to give the containr inside the dockerfile
    dockerfile_name : str
        a formatted string that defines the name of the docker file for node

    Methods
    -------
    get_dockerfile_content()
        Gets the neccessary content for the dockerfile for node

    get_docker_compose_content()
        Gets to the docker compose content for node
    """

    def __init__(self, prefix_for_containers: str):
        """
        Parameters
        ----------
        prefix : str
            The prefix applied to all container names
        container_name : str
            the name to give the containr inside the dockerfile
        dockerfile_name : str
            a formatted string that defines the name of the docker file for node
        """
        self.prefix: str = prefix_for_containers
        self.container_name: str = prefix_for_containers + '_node'
        self.dockerfile_name: str = 'node.dockerfile'

    def get_dockerfile_content(self):
        """
        Writes the neccessary content to the dockerfile for node

        """

        dockerfile_content: List[str] = [
            'FROM node:10.15.3',
            '# Update and install required packages',
            'RUN apt-get update',
            '',
            'RUN     yes | npm i pm2 -g'
        ]
        return dockerfile_content

    def get_docker_compose_content(self):
        """
        Returns the neccessary content for the docker-compose.yml file for node

        """

        docker_compose_content: List[str] = [
            "  node:",
            "    container_name: {}".format(self.container_name),
            "    build:",
            "      context: .",
            "      dockerfile: .docker/{}".format(self.dockerfile_name),
            "    volumes:",
            "      - ./src:/var/www/src",
            "    working_dir: /var/www/src",
            "    command: bash -c 'You can build and start your server here'",
            "    networks:",
            "      - {}-network".format(self.prefix)
        ]
        return docker_compose_content


from project import Project
import unittest
from unittest.mock import patch
from unittest import mock
from unittest import TestCase
import os
from pprint import pprint
import shutil
from containers.nginx import Nginx
from containers.phpfpm import Phpfpm
from containers.node import Node
from containers.python import Python
from containers.sql import Sql
from containers.mongo import Mongo
from containers.mongoseeder import MongoSeeder
from containers.apache import Apache
from containers.deno import Deno
from containers.redis import Redis
os.environ['ENV'] = 'testing'

class ProjectTest(unittest.TestCase):

    containers = 'NGINX phpfpm'

    all_ok = 'y'

    @patch('builtins.input', lambda *args: 'test-name')
    def test_set_path(self):
        project = Project()
        project.set_path()
        self.assertEqual(project.name, 'test-name')
        self.assertEqual(project.path, os.getcwd() + '/' + 'test-name')

    # Correct input for containers and all is ok
    @patch('builtins.input', side_effect=[containers, all_ok])
    def test_get_list_of_containers_1(self, mock_input):
        project = Project()
        project.get_list_of_containers()
        self.assertEqual(['nginx', 'phpfpm'], project.containers)

    # Correct input for containers but all not ok
    @patch('builtins.input', side_effect=[containers, 'n'])
    def test_get_list_of_containers_2(self, mock_input):
        project = Project()
        try:
            project.get_list_of_containers()
        except:
            self.assertRaises(SystemExit)

    @patch('builtins.input', lambda *args: 'my-project')
    def test_get_prefix_for_containers(self):
        project = Project()
        project.get_prefix_for_containers()
        self.assertEqual(project.container_prefix, 'my-project')

    def test_add_network_block_to_compose_file(self):
        project = Project()
        # mock the property needed in this method
        project.container_prefix = "hello"
        project.add_network_block_to_compose_file()
        # TODO :: Once implemented then assert this content is inside the file
        text_block: List[str] = [
            'network:',
            '  {}-network:'.format(self.container_prefix),
            '    driver: bridge'
        ]

    def test_init_git_repo(self):
        project = Project()
        project.init_git_repo()
        # TODO :: Assert the project is using git (check for .git folder?)

    def test_create_containers_from_container_list(self):
        # setup
        project = Project()
        project.container_prefix = "my_project"
        project.path = "./my-project"

        # nginx
        project.containers = ["nginx"]
        os.mkdir('my-project')
        os.mkdir('./my-project/.docker')
        os.mkdir('./my-project/.docker/config')
        project.create_containers_from_container_list()
        nginx = Nginx('my_project')
        # start of asserting dockerfile content
        nginx_dockerfile_content = nginx.get_dockerfile_content()
        # add nl char to list to match what is read in file
        new_dockerfile_content = []
        for x in nginx_dockerfile_content:
            new_dockerfile_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/.docker/nginx.dockerfile', 'r')
        for x in f:
          file_contents.append(x)
        self.assertEqual(new_dockerfile_content, file_contents)
        f.close()
        # start of asserting docker compose content
        nginx.depends_on_string = ''
        nginx_docker_compose_content = nginx.get_docker_compose_content()
        new_docker_compose_content = []
        for x in nginx_docker_compose_content:
            new_docker_compose_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/docker-compose.yml', 'r')
        for x in f:
          file_contents.append(x)
        self.assertEqual(new_docker_compose_content, file_contents)
        f.close()
        # start of asserting config file content
        nginx_config_file_content = nginx.get_config_file_content()
        new_config_file_content = []
        for x in nginx_config_file_content:
            new_config_file_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/.docker/config/nginx.conf', 'r')
        for x in f:
          file_contents.append(x)
        self.assertEqual(new_config_file_content, file_contents)
        f.close()
        shutil.rmtree('./my-project')

        # phpfpm
        phpfpm = Phpfpm('my_project')
        project.containers = ["phpfpm"]
        os.mkdir('my-project')
        os.mkdir('./my-project/.docker')
        os.mkdir('./my-project/.docker/config')
        project.create_containers_from_container_list()
        # start of dockerfile content
        phpfpm_dockerfile_content = phpfpm.get_dockerfile_content()
        new_dockerfile_content = []
        for x in phpfpm_dockerfile_content:
            new_dockerfile_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/.docker/phpfpm.dockerfile', 'r')
        for x in f:
          file_contents.append(x)
        self.assertEqual(new_dockerfile_content, file_contents)
        f.close()
        # start of docker compose content
        phpfpm_docker_compose_content = phpfpm.get_docker_compose_content()
        new_docker_compose_content = []
        for x in phpfpm_docker_compose_content:
            new_docker_compose_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/docker-compose.yml', 'r')
        for x in f:
          file_contents.append(x)
        self.assertEqual(new_docker_compose_content, file_contents)
        f.close()
        #start of php ini copy
        phpfpm.create_php_ini_file(project.path)
        self.assertEqual(True, os.path.isfile("./my-project/.docker/config/php.ini"))
        shutil.rmtree('./my-project')

        # node
        node = Node('my_project')
        project.containers = ["node"]
        os.mkdir('my-project')
        os.mkdir('./my-project/.docker')
        os.mkdir('./my-project/.docker/config')
        project.create_containers_from_container_list()
        # start of dockerfile content
        dockerfile_content = node.get_dockerfile_content()
        new_dockerfile_content = []
        for x in dockerfile_content:
            new_dockerfile_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/.docker/node.dockerfile', 'r')
        for x in f:
          file_contents.append(x)
        self.assertEqual(new_dockerfile_content, file_contents)
        f.close()
        # start of docker compose content
        docker_compose_content = node.get_docker_compose_content()
        new_docker_compose_content = []
        for x in docker_compose_content:
            new_docker_compose_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/docker-compose.yml', 'r')
        for x in f:
            file_contents.append(x)
        self.assertEqual(new_docker_compose_content, file_contents)
        f.close()
        shutil.rmtree('./my-project')

        # python
        python = Python('my_project')
        project.containers = ["python"]
        os.mkdir('my-project')
        os.mkdir('./my-project/.docker')
        os.mkdir('./my-project/.docker/config')
        project.create_containers_from_container_list()
        # start of dockerfile content
        dockerfile_content = python.get_dockerfile_content()
        new_dockerfile_content = []
        for x in dockerfile_content:
            new_dockerfile_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/.docker/python.dockerfile', 'r')
        for x in f:
          file_contents.append(x)
        self.assertEqual(new_dockerfile_content, file_contents)
        f.close()
        # start of docker compose content
        docker_compose_content = python.get_docker_compose_content()
        new_docker_compose_content = []
        for x in docker_compose_content:
            new_docker_compose_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/docker-compose.yml', 'r')
        for x in f:
            file_contents.append(x)
        self.assertEqual(new_docker_compose_content, file_contents)
        f.close()
        shutil.rmtree('./my-project')

        # sql
        sql = Sql('my_project')
        project.containers = ["sql"]
        os.mkdir('my-project')
        os.mkdir('./my-project/.docker')
        os.mkdir('./my-project/.docker/config')
        os.mkdir('./my-project/.docker/data')
        os.mkdir('./my-project/.docker/env')
        project.create_containers_from_container_list()
        # start of dockerfile content
        dockerfile_content = sql.get_dockerfile_content()
        new_dockerfile_content = []
        for x in dockerfile_content:
            new_dockerfile_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/.docker/sql.dockerfile', 'r')
        for x in f:
          file_contents.append(x)
        self.assertEqual(new_dockerfile_content, file_contents)
        f.close()
        # start of docker compose content
        docker_compose_content = sql.get_docker_compose_content()
        new_docker_compose_content = []
        for x in docker_compose_content:
            new_docker_compose_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/docker-compose.yml', 'r')
        for x in f:
            file_contents.append(x)
        self.assertEqual(new_docker_compose_content, file_contents)
        # created both env and dump file
        self.assertTrue(os.path.exists('./my-project/.docker/data/sql-data-dump.sql'))
        self.assertTrue(os.path.exists('./my-project/.docker/env/sql.env'))
        f.close()
        shutil.rmtree('./my-project')

        # mongo
        mongo = Mongo('my_project')
        project.containers = ["mongo"]
        os.mkdir('my-project')
        os.mkdir('./my-project/.docker')
        os.mkdir('./my-project/.docker/config')
        os.mkdir('./my-project/.docker/data')
        os.mkdir('./my-project/.docker/env')
        project.create_containers_from_container_list()
        # start of env content
        env_content = mongo.get_env_content()
        new_env_content = []
        for x in env_content:
            new_env_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/.docker/env/mongo.env', 'r')
        for x in f:
          file_contents.append(x)
        self.assertEqual(new_env_content, file_contents)
        f.close()
        # start of docker compose content
        docker_compose_content = mongo.get_docker_compose_content()
        new_docker_compose_content = []
        for x in docker_compose_content:
            new_docker_compose_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/docker-compose.yml', 'r')
        for x in f:
            file_contents.append(x)
        self.assertEqual(new_docker_compose_content, file_contents)
        shutil.rmtree('./my-project')

        # mongoseeder
        mongo_seeder = MongoSeeder('my_project')
        project.containers = ["mongoseeder"]
        os.mkdir('my-project')
        os.mkdir('./my-project/.docker')
        os.mkdir('./my-project/.docker/config')
        os.mkdir('./my-project/.docker/data')
        os.mkdir('./my-project/.docker/env')
        project.create_containers_from_container_list()
        # start of dockerfile content
        dockerfile_content = mongo_seeder.get_dockerfile_content()
        new_dockerfile_content = []
        for x in dockerfile_content:
            new_dockerfile_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/.docker/mongoseeder.dockerfile', 'r')
        for x in f:
          file_contents.append(x)
        self.assertEqual(new_dockerfile_content, file_contents)
        f.close()
        # start of docker compose content
        docker_compose_content = mongo_seeder.get_docker_compose_content()
        new_docker_compose_content = []
        for x in docker_compose_content:
            new_docker_compose_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/docker-compose.yml', 'r')
        for x in f:
            file_contents.append(x)
        self.assertEqual(new_docker_compose_content, file_contents)
        self.assertTrue(os.path.exists('./my-project/.docker/data/mongo-data-dump'))
        shutil.rmtree('./my-project')

        # apache
        apache = Apache('my_project')
        project.containers = ["apache"]
        os.mkdir('my-project')
        os.mkdir('./my-project/.docker')
        os.mkdir('./my-project/.docker/config')
        project.create_containers_from_container_list()
        # start of asserting dockerfile content
        dockerfile_content = apache.get_dockerfile_content()
        # add nl char to list to match what is read in file
        new_dockerfile_content = []
        for x in dockerfile_content:
            new_dockerfile_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/.docker/apache.dockerfile', 'r')
        for x in f:
          file_contents.append(x)
        self.assertEqual(new_dockerfile_content, file_contents)
        f.close()
        # start of asserting docker compose content
        docker_compose_content = apache.get_docker_compose_content()
        new_docker_compose_content = []
        for x in docker_compose_content:
            new_docker_compose_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/docker-compose.yml', 'r')
        for x in f:
          file_contents.append(x)
        self.assertEqual(new_docker_compose_content, file_contents)
        f.close()
        # start of asserting config file content
        config_content = apache.get_config_content()
        new_config_content = []
        for x in config_content:
            new_config_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/.docker/config/demoapache.conf', 'r')
        for x in f:
          file_contents.append(x)
        self.assertEqual(new_config_content, file_contents)
        f.close()
        shutil.rmtree('./my-project')

        # deno
        deno = Deno('my_project')
        project.containers = ["deno"]
        os.mkdir('my-project')
        os.mkdir('./my-project/.docker')
        os.mkdir('./my-project/.docker/config')
        project.create_containers_from_container_list()
        # start of dockerfile content
        dockerfile_content = deno.get_dockerfile_content()
        new_dockerfile_content = []
        for x in dockerfile_content:
            new_dockerfile_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/.docker/deno.dockerfile', 'r')
        for x in f:
          file_contents.append(x)
        self.assertEqual(new_dockerfile_content, file_contents)
        f.close()
        # start of docker compose content
        docker_compose_content = deno.get_docker_compose_content()
        new_docker_compose_content = []
        for x in docker_compose_content:
            new_docker_compose_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/docker-compose.yml', 'r')
        for x in f:
            file_contents.append(x)
        self.assertEqual(new_docker_compose_content, file_contents)
        f.close()
        shutil.rmtree('./my-project')

        # redis
        redis = Redis('my_project')
        project.containers = ["redis"]
        os.mkdir('my-project')
        os.mkdir('./my-project/.docker')
        os.mkdir('./my-project/.docker/config')
        project.create_containers_from_container_list()
        # start of dockerfile content
        dockerfile_content = redis.get_dockerfile_content()
        new_dockerfile_content = []
        for x in dockerfile_content:
            new_dockerfile_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/.docker/redis.dockerfile', 'r')
        for x in f:
          file_contents.append(x)
        self.assertEqual(new_dockerfile_content, file_contents)
        f.close()
        # start of docker compose content
        docker_compose_content = redis.get_docker_compose_content()
        new_docker_compose_content = []
        for x in docker_compose_content:
            new_docker_compose_content.append(x + '\n')
        file_contents = []
        f = open('./my-project/docker-compose.yml', 'r')
        for x in f:
            file_contents.append(x)
        self.assertEqual(new_docker_compose_content, file_contents)
        f.close()
        shutil.rmtree('./my-project')


    def test_init_docker_compose_file(self):
        project = Project()
        # TODO :: assert the below is inside the file when the method is implemented
        text: List[str] = [
            'version: "3"',
            '  services:'
        ]
        project.init_docker_compose_file()

    def test_create_base_files(self):
        project = Project()
        project.path = "./my-project"
        project.create_base_files()
        self.assertEqual(True, os.path.exists("./my-project"))
        self.assertEqual(True, os.path.exists("./my-project/LICENSE.TXT"))
        self.assertEqual(True, os.path.exists("./my-project/.gitignore"))
        self.assertEqual(True, os.path.exists("./my-project/docker-compose.yml"))
        self.assertEqual(True, os.path.exists("./my-project/src"))
        self.assertEqual(True, os.path.exists("./my-project/.docker"))
        self.assertEqual(True, os.path.exists("./my-project/.docker/config"))
        self.assertEqual(True, os.path.exists("./my-project/.docker/data"))
        self.assertEqual(True, os.path.exists("./my-project/.docker/env"))
        shutil.rmtree("./my-project")


if __name__ == '__main__':
    print('[Testing] Project: Started...')
    unittest.main()


import config

"""
...
Usage
---------
mongo = mongo('prefix name e.g. myprojectname')
mongo.write_to_*()

"""

class Mongo:
    """
    A class used to represent the mongo container

    ...

    Attributes
    ----------
    prefix : str
        The prefix applied for all container names
    container_name : str
        the name to give the containr inside the dockerfile

    Methods
    -------
    get_docker_compose_content()
        Returns to the docker compose file with the neccessary mongo content

    get_env_content(root_path: str)
        Returns the environmental file content
    """

    def __init__(self, prefix_for_containers: str):
        """
        Parameters
        ----------
        prefix : str
            The prefix applied to all container names
        container_name : str
            the name to give the containr inside the dockerfile
        dockerfile_name : str
            a formatted string that defines the name of the docker file for mongo
        """
        self.prefix: str = prefix_for_containers
        self.container_name: str = prefix_for_containers + '_mongo'

    def get_docker_compose_content(self):
        """
        Returns the neccessary content to the docker-compose.yml file for mongo

        """

        docker_compose_content: List[str] = [
            "  mongodb:",
            "    container_name: {}".format(self.container_name),
            "    image: mongo",
            "    ports:",
            '      "27017:27017"',
            "    env_file:",
            "      - ./.docker/env/mongo.env",
            "    networks:",
            "      - {}-network".format(self.prefix)
        ]
        return docker_compose_content

    def get_env_content(self):
        """
        Returns the environment file content

        """
        content = [
            'MONGODB_ROOT_USERNAME=root',
            'MONGODB_ROOT_PASSWD=rootpassword',
            'MONGODB_ROOT_ROLE=root',
            'MONGODB_USERNAME=user',
            'MONGODB_PASSWD=password',
            'MONGODB_DBNAME=db',
            'MONGODB_ROLE=readWrite'
        ]
        return content


import config
import os

class Phpfpm:

    """
    A class used to represent the Php-fpm container

    ...

    Attributes
    ----------
    prefix : str
        The prefix applied for all container names
    container_name : str
        the name to give the containr inside the dockerfile
    port : int
        port for phpfpm to run on
    dockerfile_name : str
        a formatted string that defines the name of the docker file for phpfpm

    Methods
    -------
    write_to_dockerfile()
        Returns the neccessary content for the dockerfile for phpfpm

    write_to_docker_compose_file()
        Returns the docker compose content for phpfpm

    create_php_ini_file()
        Copies over the php.ini file
    """

    def __init__(self, prefix_for_containers: str):
        """
        Parameters
        ----------
        prefix : str
            The prefix applied to all container names
        container_name : str
            the name to give the container inside the dockerfile
        port : int
            port for phpfpm to run on
        dockerfile_name : str
            a formatted string that defines the name of the docker file for phpfpm
        """
        self.prefix: str = prefix_for_containers
        self.container_name: str = prefix_for_containers + '_phpfpm'
        self.port: str = config.ports['phpfpm']
        self.dockerfile_name: str = 'phpfpm.dockerfile'

    def get_dockerfile_content(self):
        """
        Returns the neccessary content for the dockerfile for phpfpm

        """

        dockerfile_content: List[str] = [
            'FROM php:7.4-fpm',
            '',
            '# Update and install required packages and dependencies',
            'RUN apt-get update -y && apt-get install apt-transport-https -y',
            'RUN apt-get install vim -y',
            'RUN apt-get install apt-utils -y',
            'RUN apt-get install libc-client-dev -y',
            'RUN apt-get install libzip-dev -y',
            'RUN apt-get install -y libxml2-dev libxslt-dev python-dev --no-install-recommends',
            'RUN apt-get install -y libldb-dev libldap2-dev',
            'RUN apt-get install -y libpng-dev',
            '',
            '# Avilable extensions by default when using docker-php-ext-install',
            '# bcmath bz2 calendar ctype curl dba dom mysqli enchant exif fileinfo filter ftp gd gettext gmp hash iconv imap interbase intl json ldap mbstring mysqli oci8 odbc opcache pcntl pdo pdo_dblib pdo_firebird pdo_mysql pdo_oci pdo_odbc pdo_pgsql pdo_sqlite pgsql phar posix pspell readline recode reflection session shmop simplexml snmp soap sockets sodium spl standard sysvmsg sysvsem sysvshm tidy tokenizer wddx xml xmlreader xmlrpc xmlwriter xsl zend_test zip',
            'RUN docker-php-ext-install pdo pdo_mysql mysqli xml json ldap mbstring soap gd xsl zip sockets',
            '',
            '# Install Xdebug',
            'RUN yes | pecl install xdebug \\',
            '&& echo "[Xdebug]" > /usr/local/etc/php/conf.d/xdebug.ini \\',
            '&& echo "zend_extension=$(find /usr/local/lib/php/extensions/ -name xdebug.so)" >> /usr/local/etc/php/conf.d/xdebug.ini \\',
            '&& echo "xdebug.remote_enable=on" >> /usr/local/etc/php/conf.d/xdebug.ini \\',
            '&& echo "xdebug.remote_autostart=off" >> /usr/local/etc/php/conf.d/xdebug.ini \\',
            '&& echo "xdebug.idekey=VSCODE" >> /usr/local/etc/php/conf.d/xdebug.ini \\',
            '&& echo "xdebug.remote_host=host.docker.internal" >> /usr/local/etc/php/conf.d/xdebug.ini \\',
            '&& echo "xdebug.remote_port=9001" >> /usr/local/etc/php/conf.d/xdebug.ini \\',
            '&& docker-php-ext-enable xdebug',
            '',
            '# Configure php.ini',
            'COPY ./.docker/config/phpfpm/php.ini /etc/php.ini',
            '',
            '# Install composer',
            'RUN curl -sS https://getcomposer.org/installer | php -- --install-dir=/usr/local/bin --filename=composer'
        ]
        return dockerfile_content

    def get_docker_compose_content(self):
        """
        Returns the neccessary content for the docker-compose.yml file for phpfpm

        """

        docker_compose_content: List[str] = [
            '  phpfpm:',
            '    container_name: {}_phpfpm'.format(self.prefix),
            '    build:',
            '      context: .',
            '      dockerfile: .docker/{}'.format(self.dockerfile_name),
            '    ports:',
            '      - {}:{}'.format(self.port, self.port),
            '    volumes:',
            '      - ./src:/var/www/src',
            '    command: bash -c "echo Here you can install your composer dependencies and run PHP-FPM e.g. phpfpm -F"',
            '    working_dir: /var/www/src',
            '    networks:',
            '      - {}-network'.format(self.prefix)
        ]
        return docker_compose_content

    def create_php_ini_file(self, root_path):
        """
        Copies over the php.ini file into the users project

        """

        from shutil import copyfile
        if os.environ['ENV'] == 'testing':
            copyfile('{}/../data/php.ini'.format(root_path), '{}/.docker/config/php.ini'.format(root_path))
        else:
            copyfile('{}/denv-creator/data/php.ini'.format(root_path), '{}/.docker/config/php.ini'.format(root_path))


import config

"""
...
Usage
---------
apache = apache('prefix name e.g. myprojectname')
apache.write_to_*()

"""

class Apache:
    """
    A class used to represent the apache container

    ...

    Attributes
    ----------
    prefix : str
        The prefix applied for all container names
    container_name : str
        the name to give the containr inside the dockerfile
    port : int
        port for apache to run on
    dockerfile_name : str
        a formatted string that defines the name of the docker file for apache

    Methods
    -------
    get_dockerfile_content()
        Returns the necessary content to the dockerfile for apache

    get_docker_compose_content()
        Returns to the docker compose file with the neccessary apache content

    get_config_content()
        Writes the necessary content to the config file with php-fpm support
    """

    def __init__(self, prefix_for_containers: str):
        """
        Parameters
        ----------
        prefix : str
            The prefix applied to all container names
        container_name : str
            the name to give the containr inside the dockerfile
        port : int
            port for apache to run on
        dockerfile_name : str
            a formatted string that defines the name of the docker file for apache
        """
        self.prefix: str = prefix_for_containers
        self.container_name: str = prefix_for_containers + '_apache'
        self.port: int = config.ports['apache']
        self.dockerfile_name: str = 'apache.dockerfile'
        self.depends_on_string = ''

    def get_dockerfile_content(self):
        """
        Returns the necessary content to the dockerfile for apache

        """

        dockerfile_content: List[str] = [
            'FROM httpd:2.4',
            '',
            'RUN apt update -y',
            'COPY .docker/conf/apache.conf /usr/local/apache2/conf/demoapache.conf',
            'RUN echo "\\nInclude /usr/local/apache2/conf/demoapache.conf" >> /usr/local/apache2/conf/httpd.conf',
            'RUN cat /usr/local/apache2/conf/httpd.conf',
        ]
        return dockerfile_content

    def get_docker_compose_content(self):
        """
        Returns the neccessary content to the docker-compose.yml file for apache

        """

        docker_compose_content: List[str] = [
            "  apache:",
            "    container_name: {}".format(self.container_name),
            "    build:",
            "      context: .",
            "      dockerfile: .docker/{}".format(self.dockerfile_name),
            "    volumes:",
            "      - ./src:/var/www/src",
            "    working_dir: /var/www/src",
            "    ports:",
            "      - '{}:{}'".format(self.port, self.port),
            "    networks:",
            "      - {}-network".format(self.prefix)
        ]
        if self.depends_on_string != '':
            docker_compose_content.insert(2, self.depends_on_string)
        return docker_compose_content

    def get_config_content(self):
        """
        Returns the necessary content to the config file for apache with php-fpm support commented out
    
        """

        config_content: List[str] = [
            'LoadModule proxy_module /usr/local/apache2/modules/mod_proxy.so',
            'LoadModule proxy_http_module modules/mod_proxy_http.so',
            '',
            '<VirtualHost *:80>',
            '  ProxyPass / http://your_container_name:port/',
            '</VirtualHost>'
        ]
        return config_content

    def update_services_to_depend_on(self, depends_on_string: str):
        self.depends_on_string = depends_on_string